﻿using System.Collections.Generic;
using System.Data.OleDb;
using System.Windows.Forms;

using DevExpress.Utils;
using DevExpress.XtraEditors;
using DevExpress.XtraGrid;
using DevExpress.XtraGrid.Views.Base;
using DevExpress.XtraGrid.Views.Grid;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 연결
        /// </summary>
        private OleDbConnection connection;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.connection = new OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=nwind.mdb");

            #region 조부모 그리드 뷰를 설정한다.

            GridView grandParentGridView = new GridView();

            grandParentGridView.Name = "grandParentGridView";

            grandParentGridView.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                false                           // Editable
            );

            grandParentGridView.OptionsDetail.EnableMasterViewMode = true;

            grandParentGridView.OptionsDetail.AllowExpandEmptyDetails = false;

            grandParentGridView.OptionsBehavior.AutoPopulateColumns = true;

            #endregion
            #region 부모 그리드 뷰를 설정한다.

            GridView parentGridView = new GridView();

            parentGridView.Name = "parentGridView";

            parentGridView.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                false                           // Editable
            );

            parentGridView.OptionsBehavior.AutoPopulateColumns = true;

            #endregion
            #region 자식 그리드 뷰를 설정한다.

            GridView childGridView = new GridView();

            childGridView.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                false                           // Editable
            );

            childGridView.OptionsBehavior.AutoPopulateColumns = true;

            #endregion
            #region 그리드 레벨 노드 2를 설정한다.

            GridLevelNode gridLevelNode2 = new GridLevelNode();

            gridLevelNode2.RelationName  = "Relation2";
            gridLevelNode2.LevelTemplate = childGridView;

            #endregion
            #region 그리드 레벨 노드 1을 설정한다.

            GridLevelNode gridLevelNode1 = new GridLevelNode();

            gridLevelNode1.RelationName  = "Relation1";
            gridLevelNode1.LevelTemplate = parentGridView;

            gridLevelNode1.Nodes.Add(gridLevelNode2);

            #endregion
            #region 그리드 컨트롤을 설정한다.

            GridControl gridControl = new GridControl();

            gridControl.Name     = "gridControl";
            gridControl.Dock     = DockStyle.Fill;
            gridControl.MainView = grandParentGridView;

            gridControl.ViewCollection.AddRange(new BaseView[] { grandParentGridView, parentGridView, childGridView });

            gridControl.LevelTree.Nodes.Add(gridLevelNode1);

            Controls.Add(gridControl);

            gridControl.DataSource = Customer.GetCustomerList(this.connection);

            #endregion

            grandParentGridView.BestFitColumns();

            #region 이벤트를 설정한다.

            grandParentGridView.MasterRowExpanded         += gridView_MasterRowExpanded;
            grandParentGridView.MasterRowGetRelationCount += gridView_MasterRowGetRelationCount;
            grandParentGridView.MasterRowEmpty            += gridView_MasterRowEmpty;
            grandParentGridView.MasterRowGetRelationName  += gridView_MasterRowGetRelationName;
            grandParentGridView.MasterRowGetChildList     += gridView_MasterRowGetChildList;

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 그리드 뷰 마스터 행 확장시 처리하기 - gridView_MasterRowExpanded(sender, e)

        /// <summary>
        /// 그리드 뷰 마스터 행 확장시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridView_MasterRowExpanded(object sender, CustomMasterRowEventArgs e)
        {
            GridView gridView = sender as GridView;

            GridView childGridView = gridView.GetDetailView(e.RowHandle, e.RelationIndex) as GridView;

            if(childGridView.Name == "grandParentGridView")
            {
                childGridView.Name = "parentGridView";

                childGridView.BestFitColumns();

                childGridView.RefreshData();

                childGridView.MasterRowExpanded         += gridView_MasterRowExpanded;
                childGridView.MasterRowGetRelationCount += gridView_MasterRowGetRelationCount;
                childGridView.MasterRowEmpty            += gridView_MasterRowEmpty;
                childGridView.MasterRowGetRelationName  += gridView_MasterRowGetRelationName;
                childGridView.MasterRowGetChildList     += gridView_MasterRowGetChildList;
            }
            else if(childGridView.Name == "parentGridView")
            {
                childGridView.Name = "childGridView";

                childGridView.BestFitColumns();
            }
        }

        #endregion
        #region 그리드 뷰 마스터 행 관계 수 구하기 - gridView_MasterRowGetRelationCount(sender, e)

        /// <summary>
        /// 그리드 뷰 마스터 행 관계 수 구하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridView_MasterRowGetRelationCount(object sender, MasterRowGetRelationCountEventArgs e)
        {
            GridView gridView = sender as GridView;

            if(gridView.Name == "grandParentGridView")
            {
                e.RelationCount = 1;
            }
            else if(gridView.Name == "parentGridView")
            {
                e.RelationCount = 1;
            }
        }

        #endregion
        #region 그리드 뷰 마스터 행 세부 내역 여부 처리하기 - gridView_MasterRowEmpty(sender, e)

        /// <summary>
        /// 그리드 뷰 마스터 행 세부 내역 여부 설정하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridView_MasterRowEmpty(object sender, MasterRowEmptyEventArgs e)
        {
            e.IsEmpty = false;
        }

        #endregion
        #region 그리드 뷰 마스터 행 관계명 구하기 - gridView_MasterRowGetRelationName(sender, e)

        /// <summary>
        /// 그리드 뷰 마스터 행 관계명 구하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridView_MasterRowGetRelationName(object sender, MasterRowGetRelationNameEventArgs e)
        {
            GridView gridView = sender as GridView;

            if(gridView.Name == "grandParentGridView")
            {
                e.RelationName = "주문 내역";
            }
            else if(gridView.Name == "parentGridView")
            {
                e.RelationName = "주문 상세 내역";
            }
        }

        #endregion
        #region 그리드 뷰 마스터 행 자식 리스트 구하기 - gridView_MasterRowGetChildList(sender, e)

        /// <summary>
        /// 그리드 뷰 마스터 행 자식 리스트 구하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gridView_MasterRowGetChildList(object sender, MasterRowGetChildListEventArgs e)
        {
            GridView gridView = sender as GridView;

            if(gridView.Name == "grandParentGridView")
            {
                Customer customer = gridView.GetRow(e.RowHandle) as Customer;

                List<Order> orderList = Order.GetOrderList(this.connection, customer);

                e.ChildList = orderList;
            }
            else if(gridView.Name == "parentGridView")
            {
                Order order = gridView.GetRow(e.RowHandle) as Order;

                List<OrderDetail> orderDetailList = OrderDetail.GetOrderDetailList(this.connection, order);

                e.ChildList = orderDetailList;
            }
        }

        #endregion
    }
}